@@ -26,7 +26,7 @@ use Encode 'encode_utf8';
 
 use experimental 'postderef';
 
-our $VERSION = "v0.0.10";
+our $VERSION = "v0.0.11";
 
 use constant {
     DEFAULT_HEARTBEAT_INTERVAL        => 5,
@@ -203,11 +203,15 @@ sub register ($self, $app, $config) {
         BoardStreams::Registry->add_request($channel_name, $request_name, $request_sub);
     });
 
-    $app->helper("$bs_prefix.create_channel_or_die", sub ($c, $channel_name, $state = undef, $type = '') {
+    $app->helper("$bs_prefix.create_channel_or_die", sub ($c, $channel_name, $state = undef, $attrs = {}) {
+        my $type = $attrs->{type} // '';
+        my $keep_events = exists $attrs->{keep_events} ? to_bool $attrs->{keep_events} : true;
         my $sth;
         try {
-            $sth = $c->$bs_prefix->db->dbh->prepare('INSERT INTO "channel" (name, type, state) VALUES (?, ?, ?)');
-            $sth->execute($channel_name, $type, to_json($state));
+            $sth = $c->$bs_prefix->db->dbh->prepare('
+                INSERT INTO "channel" (name, type, state, keep_events) VALUES (?, ?, ?, ?)
+            ');
+            $sth->execute($channel_name, $type, to_json($state), $keep_events);
         } catch {
             my $err = $@;
             # look up 23505 in: https://www.postgresql.org/docs/current/errcodes-appendix.html
@@ -219,7 +223,9 @@ sub register ($self, $app, $config) {
         }
     });
 
-    $app->helper("$bs_prefix.create_channel", sub ($c, $channel_name, $state = undef, $type = '') {
+    $app->helper("$bs_prefix.create_channel", sub ($c, $channel_name, $state = undef, $attrs = {}) {
+        my $type = $attrs->{type} // '';
+        my $keep_events = exists $attrs->{keep_events} ? to_bool $attrs->{keep_events} : true;
         my $db = $c->$bs_prefix->db;
         ! $db->select('channel',
             ['id'],
@@ -227,9 +233,10 @@ sub register ($self, $app, $config) {
         )->hash or return;
         $db->insert('channel',
             {
-                name  => $channel_name,
-                type  => $type,
-                state => to_json($state),
+                name        => $channel_name,
+                type        => $type,
+                state       => to_json($state),
+                keep_events => $keep_events,
             },
             { on_conflict => undef },
         );
@@ -242,7 +249,7 @@ sub register ($self, $app, $config) {
         my $db = $c->$bs_prefix->db;
         my $tx; $tx = $db->begin unless $opts{no_txn};
         my $rows = $db->select('channel',
-            [qw/ id name state /],
+            [qw/ id name state keep_events /],
             { name => {-in => $channel_names} },
             { for => 'update' },
         )->hashes;
@@ -314,7 +321,7 @@ sub register ($self, $app, $config) {
         while( my ($row, $answer, $orig_state) = $ea->() ) {
 
             my ($event, $new_state, $guard_inc) = @$answer;
-            my ($channel_id, $channel_name) = $row->@{qw/ id name /};
+            my ($channel_id, $channel_name, $keep_events) = $row->@{qw/ id name keep_events /};
 
             {
                 my @events = ref($event) eq 'REF' ? @$$event : ($event);
@@ -323,13 +330,22 @@ sub register ($self, $app, $config) {
 
                 # for all but last event
                 foreach my $event (@events) {
-                    my ($event_id, $dt) = $db->insert('event_patch',
-                        {
-                            channel_id => $channel_id,
-                            event      => to_json($event),
-                        },
-                        { returning => ['id', 'datetime'] },
-                    )->hash->@{qw/ id datetime /};
+                    my ($event_id, $dt);
+                    if ($keep_events) {
+                        ($event_id, $dt) = $db->insert('event_patch',
+                            {
+                                channel_id => $channel_id,
+                                event      => to_json($event),
+                            },
+                            { returning => ['id', 'datetime'] },
+                        )->hash->@{qw/ id datetime /};
+                    } else {
+                        ($event_id, $dt) =
+                            $db->query(
+                                "SELECT nextval(?), current_timestamp",
+                                $event_patch_sequence_name
+                            )->array->@*;
+                    }
 
                     do_notifications($channel_name, $event_id, $event, undef);
                 }
@@ -340,7 +356,7 @@ sub register ($self, $app, $config) {
             $diff = undef if $diff and ! %$diff;
             next ANSWER unless defined $event or defined $diff;
             my ($event_id, $dt);
-            if (defined $event) {
+            if ($keep_events and defined $event) {
                 ($event_id, $dt) = $db->insert('event_patch',
                     {
                         channel_id => $channel_id,
@@ -735,6 +751,59 @@ sub register ($self, $app, $config) {
             if $duration > 1;
     });
 
+    $app->helper("$bs_prefix.delete_events", sub ($c, $channel_name, %opts) {
+        # opts can be: keep_num and/or keep_dur
+        my @until_ids;
+
+        my $channel_id = $c->$bs_prefix->db->select('channel',
+            ['id'],
+            { name => $channel_name },
+        )->hash->{id} or return;
+
+        if (defined $opts{keep_num}) {
+            my $until_hash = $c->$bs_prefix->db->select('event_patch',
+                ['id'],
+                {
+                    channel_id => $channel_id,
+                },
+                {
+                    order_by => { -desc => 'id' },
+                    offset   => $opts{keep_num},
+                    limit    => 1,
+                },
+            )->hash;
+            my $until_id = $until_hash ? $until_hash->{id} : 0;
+            push @until_ids, $until_id;
+        }
+
+        if (defined $opts{keep_dur}) {
+            $opts{keep_dur} .= ' SECOND' if $opts{keep_dur} =~ /^\d+\z/;
+            my $interval = $c->$bs_prefix->db->dbh->quote( $opts{keep_dur} );
+            my $until_hash = $c->$bs_prefix->db->select('event_patch',
+                ['id'],
+                {
+                    channel_id => $channel_id,
+                    datetime   => { '<', \"current_timestamp - INTERVAL $interval" },
+                },
+                {
+                    order_by => { -desc => 'datetime' },
+                    limit    => 1,
+                },
+            )->hash;
+            my $until_id = $until_hash ? $until_hash->{id} : 0;
+            push @until_ids, $until_id;
+        }
+
+        my $until_id = min @until_ids;
+
+        $c->$bs_prefix->db->delete('event_patch',
+            {
+                channel_id => $channel_id,
+                defined($until_id) ? (id => { '<=', $until_id }) : (),
+            }
+        );
+    });
+
     # system channels
     {
         $app->$bs_prefix->create_channel($WORKERS_CHANNEL, {});
